package de.opalpiraten.scenepainter.tools.basic;
import javax.swing.*;
import javax.swing.tree.*;
import java.awt.*;
import java.util.regex.*;
import java.util.*;
import java.awt.event.*;
import javax.swing.event.*;
import de.opalpiraten.scenepainter.gui.*;
import de.opalpiraten.scenepainter.gui.render.*;

import de.opalpiraten.scenepainter.sys.*;
import com.threerings.media.image.*;


/**
 * This tool shows the names of all components/tiles, defined in the 
 * nenyabrowser and allows the user to search for names by supplying
 * a regular expression for filtering.
 */
public class Finder extends JInternalFrame implements ActionListener,
KeyListener, InternalFrameListener {
  
  /**
   * Switch to showing composites
   */
  private JRadioButton composites = new JRadioButton(Dict.get("tools.basic.finder.composites"));
  
  /**
   * Switch to showing tiles
   */
  private JRadioButton tiles  = new JRadioButton(Dict.get("tools.basic.finder.tiles"));
  
  /**
   * Displays the list of components/tiles
   */
  private JTextArea list = new JTextArea(15,35);
  
  /**
   * User may type a regualr expression herin to filter out the desired
   * components/tiles.
   */
  private JTextField filter = new JTextField(20);
  
  /**
   * Buffer for all the component names in the nenyabrowser
   */
  private String[] cri;
  
  /**
   * Buffer for all the tilenames in the nenyabrowser
   */
  private String[] tri;
  
  /**
   * shows how many entries are displayed
   */
  private JTextField info = new JTextField(8);
  
  /**
   * Sole constructor
   */
  public Finder() {
    super(Dict.get("tools.basic.finder.title"),true,true,true,true);
    cri=Core.nenyabrowser.getComponentNames();
    tri=Core.nenyabrowser.getTileNames();
    
    ButtonGroup group = new ButtonGroup();
    group.add(tiles);
    group.add(composites);
    
    list.setEditable(false);
    filter.requestFocus();
    filter.addKeyListener(this);
    composites.addActionListener(this);
    tiles.addActionListener(this);
    tiles.setSelected(true);
    actionPerformed(null);
    
    JPanel content = new JPanel();
    //content.setLayout(new BoxLayout(content,BoxLayout.Y_AXIS));
    GridBagLayout gbl = new GridBagLayout();
    content.setLayout(gbl);
    GridBagConstraints gbc = new GridBagConstraints();
    gbc.anchor=GridBagConstraints.NORTHWEST;
    gbc.fill=GridBagConstraints.BOTH;
    gbc.gridx=0;
    gbc.gridy=0;
    gbc.gridwidth=1; 
    gbc.gridheight=1;
    gbc.weightx=1.0;
    gbc.weighty=1.0;
    gbc.insets = new Insets(1,1,1,1);
    JScrollPane jsp = new JScrollPane(list);
    gbl.setConstraints(jsp,gbc);
    content.add(jsp);

    JPanel bg = new JPanel();
    bg.add(tiles);
    bg.add(composites);
    bg.add(filter);
    bg.add(info);
    
    gbc.anchor=GridBagConstraints.NORTHWEST;
    gbc.fill=GridBagConstraints.BOTH;
    gbc.gridx=0;
    gbc.gridy=1;
    gbc.gridwidth=1; 
    gbc.gridheight=1;
    gbc.weightx=0.0;
    gbc.weighty=0.0;
    gbc.insets = new Insets(1,1,1,1);
    gbl.setConstraints(bg,gbc);
    content.add(bg);

    
    info.setEditable(false);
    setContentPane(content);
    pack();
    setLocation(Core.workspace.getPlacementHint(this));
    addInternalFrameListener(this);
    //setPreferredSize(new Dimension(500,250));
  }
  
  
  
  public void actionPerformed(ActionEvent e) {
    filter.setText("");
    StringBuffer sb = new StringBuffer();
    Integer data[] = new Integer[2];
    
    if (tiles.isSelected()) {
      for(int i=0;i<tri.length;i++) {
        sb.append(tri[i]);
        if (i<tri.length-1) sb.append("\n");
      }
      data[0]= new Integer(tri.length);
      data[1]= new Integer(tri.length);
    }
    else {
      for(int i=0;i<cri.length;i++) {
        sb.append(cri[i]);
        if (i<cri.length-1) sb.append("\n");
      }
      data[0]= new Integer(cri.length);
      data[1]= new Integer(cri.length);
    }
    info.setText(Dict.get("tools.basic.finder.info",data));
    list.setText(sb.toString());
    list.setCaretPosition(0);
  }
  
  public void keyPressed(KeyEvent e) {}
  public void keyTyped(KeyEvent e) {}
  
  public void keyReleased(KeyEvent e) {
    String expr = filter.getText();
    if (expr.equals("")) {
      actionPerformed(null);
      return;
    }
    
    try {
      StringBuffer sb = new StringBuffer();
      Pattern p = Pattern.compile(expr,Pattern.CASE_INSENSITIVE);
      Matcher m;
      Integer data[] = new Integer[2];
      
      if (tiles.isSelected()) {
        int count=0;
        for(int i=0;i<tri.length;i++) {
          m = p.matcher(tri[i]);
          if (m.find()) {
            sb.append(tri[i]);
            if (i<tri.length-1) sb.append("\n");
            count++;
          }
        }
        data[0]= new Integer(count);
        data[1]= new Integer(tri.length);
      }
      else {
        int count=0;
        for(int i=0;i<cri.length;i++) {
          m = p.matcher(cri[i]);
          if (m.find()) { 
            sb.append(cri[i]);
            if (i<cri.length-1) sb.append("\n"); 
            count++;
          }
        }
        data[0]= new Integer(count); 
        data[1]= new Integer(cri.length);
      }
      info.setText(Dict.get("tools.basic.finder.info",data));
      list.setText(sb.toString());
      list.setCaretPosition(0);
    }
    catch (Exception exp) {
      list.setText(exp.getMessage());
    }
  }
  
  public void internalFrameActivated(InternalFrameEvent e) {
    filter.requestFocus();
  }
  
  public void internalFrameClosed(InternalFrameEvent e) {}
  public void internalFrameClosing(InternalFrameEvent e) {}
  public void internalFrameDeactivated(InternalFrameEvent e) {}
  public void internalFrameDeiconified(InternalFrameEvent e) {}
  public void internalFrameIconified(InternalFrameEvent e) {}
  public void internalFrameOpened(InternalFrameEvent e) {}
 
}
